@@ -419,6 +419,64 @@ g_usb_device_open (GUsbDevice  *device,
 	return g_usb_device_libusb_error_to_gerror (device, rc, error);
 }
 
+/**
+ * g_usb_device_get_custom_index:
+ * @device: a #GUsbDevice
+ * @class_id: a device class, e.g. 0xff for VENDOR
+ * @subclass_id: a device subclass
+ * @protocol_id: a protocol number
+ * @error: a #GError, or %NULL
+ *
+ * Gets the string index from the vendor class interface descriptor.
+ *
+ * Return value: a non-zero index, or 0x00 for failure
+ *
+ * Since: 0.2.5
+ **/
+guint8
+g_usb_device_get_custom_index (GUsbDevice *device,
+			       guint8      class_id,
+			       guint8      subclass_id,
+			       guint8      protocol_id,
+			       GError    **error)
+{
+	const struct libusb_interface_descriptor *ifp;
+	gint rc;
+	guint8 idx = 0x00;
+	guint i;
+	struct libusb_config_descriptor *config;
+
+	rc = libusb_get_active_config_descriptor (device->priv->device, &config);
+	if (!g_usb_device_libusb_error_to_gerror (device, rc, error))
+		return NULL;
+
+	/* find the right data */
+	for (i = 0; i < config->bNumInterfaces; i++) {
+		ifp = &config->interface[i].altsetting[0];
+		if (ifp->bInterfaceClass != class_id)
+			continue;
+		if (ifp->bInterfaceSubClass != subclass_id)
+			continue;
+		if (ifp->bInterfaceProtocol != protocol_id)
+			continue;
+		idx = ifp->iInterface;
+		break;
+	}
+
+	/* nothing matched */
+	if (idx == 0x00) {
+		g_set_error (error,
+			     G_USB_DEVICE_ERROR,
+			     G_USB_DEVICE_ERROR_NOT_SUPPORTED,
+			     "no vendor descriptor for class 0x%02x, "
+			     "subclass 0x%02x and protocol 0x%02x",
+			     class_id, subclass_id, protocol_id);
+	}
+
+	libusb_free_config_descriptor (config);
+	return idx;
+}
+
 /**
  * g_usb_device_close:
  * @device: a #GUsbDevice
@@ -742,7 +800,7 @@ g_usb_device_control_transfer (GUsbDevice            *device,
 	GUsbSyncHelper helper;
 
 	helper.ret = -1;
-	helper.context = g_main_context_new ();
+	helper.context = g_usb_context_get_main_context (device->priv->context);
 	helper.loop = g_main_loop_new (helper.context, FALSE);
 	helper.error = error;
 	helper.finish_func = g_usb_device_control_transfer_finish;
@@ -805,7 +863,7 @@ g_usb_device_bulk_transfer (GUsbDevice    *device,
 	GUsbSyncHelper helper;
 
 	helper.ret = -1;
-	helper.context = g_main_context_new ();
+	helper.context = g_usb_context_get_main_context (device->priv->context);
 	helper.loop = g_main_loop_new (helper.context, FALSE);
 	helper.error = error;
 	helper.finish_func = g_usb_device_bulk_transfer_finish;
@@ -863,7 +921,7 @@ g_usb_device_interrupt_transfer	(GUsbDevice    *device,
 	GUsbSyncHelper helper;
 
 	helper.ret = -1;
-	helper.context = g_main_context_new ();
+	helper.context = g_usb_context_get_main_context (device->priv->context);
 	helper.loop = g_main_loop_new (helper.context, FALSE);
 	helper.error = error;
 	helper.finish_func = g_usb_device_interrupt_transfer_finish;
@@ -1555,7 +1613,7 @@ g_usb_device_get_manufacturer_index (GUsbDevice *device)
  * g_usb_device_get_device_class:
  * @device: a #GUsbDevice
  *
- * Gets the device class.
+ * Gets the device class, typically a #GUsbDeviceClassCode.
  *
  * Return value: a device class number, e.g. 0x09 is a USB hub.
  *
